local CombatLog_Object_IsA = CombatLog_Object_IsA
local COMBATLOG_FILTER_ME = COMBATLOG_FILTER_ME
local PlaySoundFile = PlaySoundFile
local GetRandomArgument = GetRandomArgument
local UnitRace = UnitRace

local canPlaySound = true

-- Combat events
local GoblinMod_CombatFrame = CreateFrame("Frame", "GM_CombatFrame")
GoblinMod_CombatFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
GoblinMod_CombatFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
GoblinMod_CombatFrame:RegisterEvent("UNIT_SPELLCAST_START")
GoblinMod_CombatFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")

-- Trade events
local GoblinMod_TradeFrame = CreateFrame("Frame", "GM_TradeFrame")
GoblinMod_TradeFrame:RegisterEvent("TRADE_SHOW")
GoblinMod_TradeFrame:RegisterEvent("TRADE_CLOSED")

-- Timer
local waitTable = {};
local waitFrame = nil;
function Call__wait(delay, func, ...)
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

function enableSound()
    canPlaySound = true
end

-- Called on combat event
function ProcessCombat(self, event, ...)
    
    soundList = {}
    
    if( event == "UNIT_SPELLCAST_SUCCEEDED") then
        local unitID, spellName, rank, lineID, spellID = ...
        if unitID ~= "player" then return end
        
        -- On successful cast
        for _,r in pairs(goblinmod_rules) do
            if( r["spellsuccess"] ~= nil ) then
                for __,s in pairs(r["spellsuccess"]) do
                    if (spellName == s) then
                        table.insert(soundList, r["soundfile"])
                    end
                end
            end
            if( r["spellsuccess_match"] ~= nil ) then
                for __,s in pairs(r["spellsuccess_match"]) do
                    if (string.match(spellName, s)) then
                        table.insert(soundList, r["soundfile"])
                    end
                end
            end
        end
        
    elseif (event == "UNIT_SPELLCAST_START" or event == "UNIT_SPELLCAST_CHANNEL_START") then
        local unitID, spellName, rank, lineID, spellID = ...
        if unitID ~= "player" then return end
        -- On begin cast
        for _,r in pairs(goblinmod_rules) do
            if( r["spellbegin"] ~= nil ) then
                for __,s in pairs(r["spellbegin"]) do
                    if (spellName == s) then
                        table.insert(soundList, r["soundfile"])
                    end
                end
            end
            if( r["spellbegin_match"] ~= nil ) then
                for __,s in pairs(r["spellbegin_match"]) do
                    if (string.match(spellName, s)) then
                        table.insert(soundList, r["soundfile"])
                    end
                end
            end
        end
            
    elseif (event == "COMBAT_LOG_EVENT_UNFILTERED") then
        local timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, destName, destFlags, destFlags2, spellId, spellName, spellSchool, amount, overkill, school, blocked, absorbed, critical, glancing, crushing = ...
        -- On buff applied
        if( CombatLog_Object_IsA(sourceFlags, COMBATLOG_FILTER_ME) and type == "SPELL_AURA_APPLIED" ) then
            
            for _,r in pairs(goblinmod_rules) do
                if( r["auras"] ~= nil ) then
                    for __,s in pairs(r["auras"]) do
                        if (spellName == s) then
                            table.insert(soundList, r["soundfile"])
                        end
                    end
                end
                if( r["auras_match"] ~= nil ) then
                    for __,s in pairs(r["auras_match"]) do
                        if (string.match(spellName, s)) then
                            table.insert(soundList, r["soundfile"])
                        end
                    end
                end
            end
            
        end
    end

    -- Play the sound(s)
    if next(soundList) ~= nil and canPlaySound then
        PlaySoundFile( GetRandomArgument (unpack(soundList)))
        canPlaySound = false
        Call__wait(1.0, enableSound)
    end
end

-- Called on trade event
function ProcessTrade(self, event, ...)
    -- Ahh potential customer!
    if( event == "TRADE_SHOW") then
    PlaySoundFile( GetRandomArgument (
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCVendor01.ogg",
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCVendor02.ogg",
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCGreeting01.ogg",
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCGreeting02.ogg",
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCGreeting03.ogg",
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCGreeting04.ogg",
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCGreeting05.ogg",
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCGreeting06.ogg",
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCPissed02.ogg"
    ))
    end
    
    -- Farewell
    if( event == "TRADE_CLOSED") then
    PlaySoundFile( GetRandomArgument (
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCFarewell01.ogg",
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCFarewell02.ogg",
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCFarewell03.ogg",
        "Sound/creature/GoblinMaleZanyNPC/GoblinMaleZanyNPCFarewell04.ogg"
    ))
    end

end

-- Registers the mod if player is a Goblin
if( UnitRace("player") == "Goblin" ) then
    print("|cFF267b26Goblin mode engaged!")
    GoblinMod_CombatFrame:SetScript("OnEvent", ProcessCombat)
    GoblinMod_TradeFrame:SetScript("OnEvent", ProcessTrade)
end

-- Slash command
SLASH_GOBLINMOD1 = '/goblinmod'
local function slashcmdhandler(msg, editbox)
    local command, arg = msg:match("^(%S*)%s*(.-)$")
    
    if(command == "play") then
        if(goblinmod_rules[arg] ~= nil and canPlaySound) then
            PlaySoundFile( goblinmod_rules[arg]["soundfile"])
            canPlaySound = false
            Call__wait(1, enableSound)
        end
    end
end
SlashCmdList["GOBLINMOD"] = slashcmdhandler